#! /bin/sh
#
# Configurator script to automatically generate conf files for MMC.
#

if test "x$1" = x; then
    prefix=`pwd`
else 
    prefix=$1
fi

echo "Configurator for Microbrew Message Center."
echo
echo "Installing files in ${prefix}"
echo
echo "If that is not good for you, please press exit and re-run the script"
echo "as follows:"
echo
echo "contrib/mmcconf.sh <absolute path to install>"
echo
echo

echo "Would you like to configure mmc now [Y/n]?"
read installnow
if test "x${installnow}" = xn -o "x${installnow}" = xN; then
    exit 0
fi

echo "Enter MySQL database server [localhost]:"
read dbserver
if test "x$dbserver" = x; then
    dbserver="localhost"
fi
echo

echo "Enter MySQL user to access from MMC [root]:"
read dbusername
if test "x$dbusername" = x; then
    dbusername="root"
fi
echo

echo "Enter password for supplied username []:"
read dbpasswd
echo

clear
echo "You can leave the following blank if this system is not a mail server."
echo

echo "Enter MMC name for this mailserver, usually an FQDN []:"
read myhostname
echo

echo "Enter username to write mailboxes as [smmsp]:"
read delivery
if test "x$delivery" = x; then
    delivery="smmsp"
fi
echo

echo "Enter UID for user '$delivery' [25]:"
read mailboxuid
if test "x$mailboxuid" = x; then
    mailboxuid="25"
fi
echo

echo "Enter UID for user '$delivery' [25]:"
read mailboxgid
if test "x$mailboxgid" = x; then
    mailboxgid="25"
fi
echo

clear
echo "You can leave the following blank if this system is not a DNS server."
echo

echo "Enter name of primary name server for the domain [${myhostname}]:"
read nsserver
if test "x$nsserver" = x; then
    nsserver=${myhostname}
fi
echo

echo "Enter the email address for the domain administrator []:"
read hostmaster
echo

mkdir ${prefix}

cat <<EOF > ${prefix}/db.conf
DBSERVER	${dbserver}
USERNAME	${dbusername}
PASSWORD	${dbpasswd}
EOF

cat <<EOF > ${prefix}/mailsrv.conf
HOSTNAME	${myhostname}
DELIVERY	${delivery}
MAILBOXUID	${mailboxuid}
MAILBOXGID	${mailboxgid}
EOF

cat <<EOF > ${prefix}/ns.conf
TTL	3600
PRINS	${nsserver}
HOSTMASTER	${hostmaster}
REFRESH	3600
RETRY	1800
EXPIRE	86400
MINIMUM	3600
STUBFILE	${prefix}/ns.stub
EOF

cat <<EOF > ${prefix}/popb4relay.conf
127.0.0.1	ALLOW
EOF

cat <<EOF > ${prefix}/ipdommap.conf
# IP Address	Domain name Mapping
# ==========	===================
127.0.0.1	${myhostname}
EOF

cat <<EOF > ${prefix}/oneuser.conf
127.0.0.1	ALLOW
EOF

touch ${prefix}/ns.stub

echo "Please examine the following files to make sure the options are as you"
echo "want them:"
echo
echo "${prefix}/db.conf"
echo "${prefix}/mailsrv.conf"
echo "${prefix}/ns.conf"
echo "${prefix}/popb4relay.conf"
echo "${prefix}/ipdommap.conf"
echo "${prefix}/oneuser.conf"
