/**********************************************************************
$Id: genzone.c,v 1.2 2002/09/08 09:25:56 bu Exp $

Copyright 2001-2002, Microbrew Software

Part of Microbrew Message Center
Multi-server Mailer Suite

http://www.microbrew.org/products/mmc/

by Bulent Yilmaz <bu@microbrew.org>
and James Brown <jim@microbrew.org>

Microbrew Message Center is free software; you can redistribute it and/or  
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

Microbrew Message Center is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Microbrew Message Center; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

**********************************************************************/

#include <stdio.h>
#include <mysql/mysql.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

/* Configuration file from autoconf. */
#include "../config.h"

/* Local library. */
#include "confread.h"

#define CONFFILE PREFIX "/etc/ns.conf"
#define DBCONF PREFIX "/etc/db.conf"
#define ZONEFILE PREFIX "/etc/ns.zone"

int gensoa(FILE *outfile) {

    char *holder, *myptr;
    FILE *stubfile;
    char readin[1024];
    
    fprintf(outfile, "; DO NOT EDIT THIS FILE!  It will be overwritten.\n");

    if((holder=readconf(CONFFILE, "TTL")) != NULL) {
        fprintf(outfile, "$TTL %s\n", holder);
        free(holder);
    } else {
        fprintf(outfile, "$TTL 3600\n");
    }
    
    if((holder=readconf(CONFFILE, "PRINS")) != NULL) {
        fprintf(outfile, "@			IN	SOA	%s.", holder);
        free(holder);
    } else {
        puts("'PRINS' not defined in " CONFFILE);
        return 1;
    }
    
    if((holder=readconf(CONFFILE, "HOSTMASTER")) != NULL) {
        if(( myptr = strchr(holder, '@') ) != NULL ) *myptr='.';
        fprintf(outfile, "	%s. (\n", holder);
        free(holder);
    } else {
        puts("'HOSTMASTER' not defined in " CONFFILE);
        return 1;
    }
    
    fprintf(outfile, "					%ld	;serial\n", time(NULL));

    if((holder=readconf(CONFFILE, "REFRESH")) != NULL) {
        fprintf(outfile, "					%s	;refresh\n", holder);
        free(holder);
    } else {
        fprintf(outfile, "					3600	;refresh\n");
    }
    
    if((holder=readconf(CONFFILE, "RETRY")) != NULL) {
        fprintf(outfile, "					%s	;retry\n", holder);
        free(holder);
    } else {
        fprintf(outfile, "					1800	;retry\n");
    }
    
    if((holder=readconf(CONFFILE, "EXPIRE")) != NULL) {
        fprintf(outfile, "					%s	;expire\n", holder);
        free(holder);
    } else {
        fprintf(outfile, "					86400	;expire\n", holder);
    }
    
    if((holder=readconf(CONFFILE, "MINIMUM")) != NULL) {
        fprintf(outfile, "					%s )	;minimum\n", holder);
        free(holder);
    } else {
        fprintf(outfile, "					%s )	;minimum\n", holder);
    }

    if((holder=readconf(CONFFILE, "STUBFILE")) != NULL) {
        fprintf(outfile, "\n; Included from %s\n", holder);
        if((stubfile = fopen(holder, "r")) != NULL) {
            while(fgets(readin, 1024, stubfile)) fputs(readin, outfile);
            fclose(stubfile);
        }
        free(holder);
        fprintf(outfile, "; End of included file\n\n");
    }

    return 0;
}

int main() {

    MYSQL *dbs=NULL;
    MYSQL_RES *result=NULL;
    MYSQL_ROW myrow=NULL;
    char **servers=NULL;
    char **ips=NULL;
    char *dotmark=NULL;
    char *tempholder=NULL;
    int i=0,j=0;
    char buildquery[2048];
    char salt[3];
    char *dbfilename=NULL;
    struct stat filestat;
    char filename[2048];
    FILE *zonefile;
    char *mysqlhost, *mysqluser, *mysqlpass;
    char *cleanholder;
    
    if((mysqlhost = readconf(DBCONF, "DBSERVER")) == NULL) {
        printf("Could not read 'DBSERVER' from %s.\n", DBCONF);
        return 1;
    }
    
    if((mysqluser = readconf(DBCONF, "USERNAME")) == NULL) {
        printf("Could not read 'USERNAME' from %s.\n", DBCONF);
        return 2;
    }
    
    if((mysqlpass = readconf(DBCONF, "PASSWORD")) == NULL) {
        printf("Could not read 'PASSWORD' from %s.\n", DBCONF);
        return 3;
    }
    
    if((dbs = mysql_init(NULL))!=NULL) {
        if(mysql_real_connect(dbs, mysqlhost, mysqluser, mysqlpass, NULL, 0, NULL, 0)) {
            if(!mysql_select_db(dbs, "mmcadmin")) {
                if(!mysql_query(dbs, "SELECT hostname, ipaddr FROM mailservers")) {
                    result = mysql_use_result(dbs);
                    
                    while(myrow = mysql_fetch_row(result)) {
                        cleanholder=servers;
                        servers=realloc(servers, ((i+1)*sizeof(char *)) );
                        if(servers != NULL) *(servers+i)=strdup(myrow[0]);
                        else {
                            puts("realloc() failed for server list.");
                            if(cleanholder != NULL) free(cleanholder);
                            return 64;
                        }
                        
                        cleanholder=ips;
                        ips=realloc(ips, ((i+1)*sizeof(char *)) );
                        if(ips != NULL) *(ips+i)=strdup(myrow[1]);
                        else {
                            puts("realloc() failed for ip list.");
                            if(cleanholder != NULL) free(cleanholder);
                            return 64;
                        }
                        
                        i++;
                    }
                    
                    if(servers != NULL) {
                        
                        if((zonefile = fopen(ZONEFILE, "w+")) != NULL) {
                            
                            if(!gensoa(zonefile)) {
                            
                                if(!mysql_query(dbs, "SELECT pdomain, server FROM customers")) {
                                    result = mysql_use_result(dbs);
                                    
                                    while(myrow = mysql_fetch_row(result)) {
                                        for(j=0;j<i;j++) {
                                            if( !strcmp(myrow[1],*(servers+j)) ) {
                                                tempholder=strdup(myrow[0]);
                                                while( (dotmark=strchr(tempholder, '.')) != NULL ) *dotmark = '-';
                                                fprintf(zonefile,"%s		IN	A	%s\n", tempholder, *(ips+j));
                                                free(tempholder);
                                                break;
                                            }
                                        }
                                    }
                                    
                                } else puts("Customer query failed.");

                            } else puts("Could not properly generate SOA record.");

                            fclose(zonefile);

                        } else puts("zonefile '" ZONEFILE "' open failed.");
                    } else puts("No domains found.");
                } else puts("Customer lookup failed.");
            } else puts("Could not select the 'mmcadmin' database.");
        } else puts("Could not connect to the database.");
    
        mysql_close(dbs);    
    } else puts("Could not allocate memory for connect.");

    return 0;
}