/**********************************************************************
$Id: popb4relayd.c,v 1.2 2002/09/08 09:25:56 bu Exp $

Copyright 2001-2002, Microbrew Software

Part of Microbrew Message Center
Multi-server Mailer Suite

http://www.microbrew.org/products/mmc/

by Bulent Yilmaz <bu@microbrew.org>
and James Brown <jim@microbrew.org>

Microbrew Message Center is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

Microbrew Message Center is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Microbrew Message Center; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

**********************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#define _REENTRANT
#include <pthread.h>

/* Configuration file from autoconf. */
#include "../config.h"

#ifndef TESTOUTPUT
#define TESTOUTPUT 1
#endif

#define DECAYTIME 900 /* 15 minutes */

#define SERV_TCP_PORT   4001
#define SERV_HOST_ADDR  "127.0.0.1"

#define NUMTHREADS 6

#define CONFFILE PREFIX "/etc/popb4relay.conf"

struct decaytable {
    long inserttime;
    unsigned long int ipaddr;
    struct decaytable *next;
    struct decaytable *prev;
};

struct sockfifo {
    int fd;
    struct sockfifo *next;
};

int dumptable(struct decaytable *listpointer) {

    struct decaytable *myentry=NULL;
    int count=1;

    myentry=listpointer;
    
    while(myentry != NULL) {
        printf("Entry %d: (%ld) %x\n", count, myentry->inserttime, myentry->ipaddr);
        myentry=myentry->next;
        count++;
    }
    
    return 0;
}

struct decaytable *first=NULL, *last=NULL;
pthread_mutex_t iplistmutex;
struct sockfifo *head=NULL, *tail=NULL;
pthread_mutex_t connmutex;

pthread_cond_t connect_cv;

long *allowedips=NULL;
int numallowedips=0;

void *decay(void *arg) {

    struct decaytable *myentry=NULL;
    char hashcommand[2047];
    struct in_addr byteaddr; 
    
    while(1) {
    
        pthread_mutex_lock(&iplistmutex);
        myentry=first;
        
        while(myentry != NULL) {
            if((myentry->inserttime + DECAYTIME) < (long)time(NULL)) {
#if TESTOUTPUT
                printf("Removing '%x'\n", myentry->ipaddr);
#endif      
                byteaddr.s_addr=myentry->ipaddr;
                
                snprintf(hashcommand, 2047, "%s -x hash /etc/mail/relaylist %s", EDITMAP_BIN, inet_ntoa(byteaddr));
                system(hashcommand);
                
                first=myentry->next;
                if(first != NULL) first->prev=NULL;
                else last=NULL;
                
                free(myentry);
                myentry=first;
#if TESTOUTPUT
                puts("Entry removed."); 
#endif            
            } else myentry=NULL;
        }
        pthread_mutex_unlock(&iplistmutex);
        
        sleep(1); /* XXX */
    }
   
    pthread_exit(NULL);
    
}


void *readsocket(void *arg) {

    char readbuf[2047];
    int readsize=0;
    struct in_addr byteaddr;
#if !(HAVE_INET_ATON)
    in_addr_t byteaddr_n;
#endif
    char hashcommand[2048];
    int exists;
    struct decaytable *mine=NULL, *traverse=NULL;
    int sockfd;
    struct sockfifo *holder;
    
    while(1) {
        
        pthread_mutex_lock(&connmutex);
        
        while(head==NULL) {
            /* Wait for a condition signal. */
#if TESTOUTPUT
            puts("Waiting for signal for 'connect_cv'.");
#endif            
            pthread_cond_wait(&connect_cv, &connmutex);
        }

#if TESTOUTPUT        
        printf("Do I have a socket? (%d)\n", (int)arg);
#endif    
        sockfd=head->fd;
        holder=head->next;
        free(head);
        head=holder;
        if(head == NULL) tail=NULL;
        pthread_mutex_unlock(&connmutex);
#if TESTOUTPUT            
        puts("Unlocked mutex 'connmutex' in readsocket."); 
#endif
        
        if((readsize=read(sockfd, readbuf, 2047)) > 0) {
            readbuf[readsize] = '\0';
#if TESTOUTPUT
            printf("Read %d bytes: %s\n", readsize, readbuf);
#endif      
            close(sockfd);
#if HAVE_INET_ATON            
            if(inet_aton(readbuf, &byteaddr)) {
#else
            if((byteaddr_n=inet_network(readbuf))!=0) {
#endif            
                traverse = first;
                exists = 0;
                
                pthread_mutex_lock(&iplistmutex);
                
                while( (traverse!=NULL) && (exists==0) ) {
#if HAVE_INET_ATON                
                    if(byteaddr.s_addr == traverse->ipaddr) {
#else
                    if(byteaddr_n == traverse->ipaddr) {
#endif                                        
                        puts("Entry already exists");
                        exists = 1;
                    } else traverse = traverse->next;
                }
                
                if(!exists) {
                    mine = (struct decaytable *)malloc(sizeof(struct decaytable));

                    if(mine != NULL) {
                        mine->inserttime=(long)time(NULL);
#if HAVE_INET_ATON
                        mine->ipaddr=byteaddr.s_addr;
#else
                        mine->ipaddr=byteaddr_n;
#endif                                                
                        
                        mine->next=NULL;
                    
                        if(last != NULL) {
                            mine->prev=last;
                            last->next=mine;
                            last=mine;
                        } else {
                            if(first == NULL) {
                                mine->prev=NULL;
                                first=mine;
                                last=mine;
                            } else {
#if TESTOUTPUT
                                puts("I lost my last pointer.  Traversing to find it. ");
#endif
                                /* Try to find my lost 'last' pointer. */

                                last = first;
                                while(last->next != NULL) last=last->next;
                                
                                /* Now link my data in. */
                                mine->prev=last;
                                last->next=mine;
                                last=mine;
                            }
                        }

                        /* Throw it in the hash table. */

#if !(HAVE_INET_ATON)
                        byteaddr.s_addr=byteaddr_n;
#endif                        
                        snprintf(hashcommand, 2047, "%s -u hash /etc/mail/relaylist %s RELAY", EDITMAP_BIN, inet_ntoa(byteaddr));
                        system(hashcommand);
                        
                        /* dumptable(first); */
                    } else puts("Malloc failed.");
                } else {
                    traverse->inserttime=(long)time(NULL);
                
                    /* Do the transplantation of this entry. */
                    
                    if(traverse != last) {
                    
                        if(traverse == first) {
                            puts("Entry is first");
                            
                            first=traverse->next;
                        } else traverse->prev->next=traverse->next;
                        
                        traverse->next->prev=traverse->prev;
                        
                        last->next=traverse;
                        
                        traverse->prev=last;
                        traverse->next=NULL;
                        last=traverse;
                        
                    } else {
#if TESTOUTPUT                    
                        puts("Entry is already the last.");
#endif                        
                    }
                }
                
                pthread_mutex_unlock(&iplistmutex);
#if TESTOUTPUT                    
                printf("Unlocked mutex 'iplistmutex' in readsocket %d.\n",(int)arg);
#endif                    
            } else {
#if TESTOUTPUT
                puts("Input was not an IP address.");
#endif            
            }
        } else {
#if TESTOUTPUT
            puts("No read done.");
#endif      
            close(sockfd);
        }
    }
        
    puts("Exiting thread");
    pthread_exit(NULL);
}

void *listensock(void *arg) {

    int sockfd, newsockfd;
    struct sockaddr_in cli_addr, serv_addr, conn_addr;
    int clilen, i, found;
    struct sockfifo *newsock;

    /* Open up socket for listening. */
    
    if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        puts("Couldn't open socket for listening.");
        pthread_exit((void *)1);
    }
    
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    serv_addr.sin_port = htons(SERV_TCP_PORT);
    
    if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
        puts("Couldn't bind to socket.");
        pthread_exit((void *)1);
        puts("I should have exited but I didn't.");
    }
    
    listen(sockfd, 5);
    
    while(1) {
#if TESTOUTPUT    
        printf("Waiting for connection.\n");
        fflush(stdout);
#endif        
        clilen = sizeof(cli_addr);
        newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
        
        if (newsockfd < 0) {
            puts("Server accept error");
        } else {
            found=0;
            i=0;
            if(!getpeername(newsockfd, (struct sockaddr *) &conn_addr, &clilen)) {
            
                while((i<numallowedips)&&(!found)) {
                    if(conn_addr.sin_addr.s_addr==allowedips[i]) found=1;
                    i++;
                }
                
                if(found) {
                
                    pthread_mutex_lock(&connmutex);
                    newsock=(struct sockfifo *)malloc(sizeof(struct sockfifo));
                    newsock->fd=newsockfd;
                    newsock->next=NULL;
                    if(tail != NULL) tail->next=newsock;
                    else head=newsock;
                    tail=newsock;
                    
                    /* Signal conditional wait and unlock mutex. */
                    
                    pthread_cond_signal(&connect_cv);
                    pthread_mutex_unlock(&connmutex);
#if TESTOUTPUT
                    puts("Unlocked mutex 'connmutex' in listener");
#endif            
                } else {
                    close(newsockfd);
#if TESTOUTPUT
                    printf("IP %x is not allowed to connect.\n", conn_addr.sin_addr.s_addr);
#endif                                        
                }
            } else {
                close(newsockfd);
#if TESTOUTPUT
                printf("Could not determine remote side IP.\n");
#endif                
            }
        }
    }
    
    pthread_exit(NULL);

}

int loadconf() {

    /* Returns 0 upon load, returns non-zero upon failure. */
    
    FILE *conffile=NULL;
    char confline[256];
    char *ip, *action;
#if HAVE_INET_ATON
    struct in_addr byteaddr;
#else
    in_addr_t byteaddr;
#endif
    int lineno=0;
    long *saver=NULL;
    
    conffile=fopen(CONFFILE, "r");
    
    if(conffile) {
        do {
            fgets(confline, 255, conffile);
            lineno++;
            
            if(confline[0] != '#') {
                if((ip=strtok(confline, " \t\r\n#")) != NULL) {
                    if((action=strtok(NULL, " \t\r\n#")) != NULL) {
                    
#if HAVE_INET_ATON                
                        if(inet_aton(ip, &byteaddr)) {
#else
                        if((byteaddr=inet_network(ip))!=0) {
#endif                    
                            if(!strncmp(action, "ALLOW", 6)) {
                                numallowedips++;
                                saver=allowedips;
                                if(( allowedips = (long *)realloc(allowedips, (numallowedips)*sizeof(long)) ) != NULL) {
#if HAVE_INET_ATON                            
                                    allowedips[numallowedips-1]=byteaddr.s_addr;
#else
                                    allowedips[numallowedips-1]=byteaddr;
#endif                                
                                } else {
                                    perror("loadconf: realloc() failed");
                                    if(saver!=NULL) free(saver);
                                    return 1;
                                }
                            } else if(!strncmp(action, "RELAY", 6)) {
                                puts("RELAY not currently supported.");
                            
                            } else printf("Warn: Invalid keyword in '%s' on line %d.\n", CONFFILE, lineno);
                        } else printf("Warn: Invalid IP in '%s' on line %d.\n", CONFFILE, lineno);
                    } else printf("Warn: Invalid format in '%s' on line %d.\n", CONFFILE, lineno);
                } else printf("Warn: Invalid format in '%s' on line %d.\n", CONFFILE, lineno);
            }
        } while(!feof(conffile));
        
        fclose(conffile);
    } else return 1;
    
    return 0;
}

int main(int argc, char *argv[]) {

    pid_t dchildpid;
    int datainput=0;
    int status;

    pthread_t threadid[NUMTHREADS];
    int i;
            
    /* First load configuration. */
    if(loadconf()) {
        puts("Failed to load a valid configuration.");
        return 1;
    }

    /* Run as a daemon */
    dchildpid = fork();
    
    if(dchildpid != 0) {
        printf("Daemon started as PID %d\n", (int)dchildpid);
        return 0;
    }

    /* First initialize my mutexes */
    
    pthread_mutex_init(&connmutex, NULL);
    pthread_mutex_init(&iplistmutex, NULL);
    
    /* Initialize conditional variable. */
    
    pthread_cond_init(&connect_cv, NULL);
    
    /* Spawn my threads. */
    
    /* Port listener. */
    pthread_create(&threadid[0], NULL, listensock, (void *)0 );
#if TESTOUTPUT    
    printf("Thread 0 created.\n");
    fflush(stdout);
#endif    
    /* Four port readers. */
    pthread_create(&threadid[1], NULL, readsocket, (void *)1 );
#if TESTOUTPUT
    printf("Thread 1 created.\n");
    fflush(stdout);
#endif    
    pthread_create(&threadid[2], NULL, readsocket, (void *)2 );
#if TESTOUTPUT        
    printf("Thread 2 created.\n");
    fflush(stdout);
#endif    
    pthread_create(&threadid[3], NULL, readsocket, (void *)3 );
#if TESTOUTPUT    
    printf("Thread 3 created.\n");
    fflush(stdout);
#endif    
    pthread_create(&threadid[4], NULL, readsocket, (void *)4 );
#if TESTOUTPUT        
    printf("Thread 4 created.\n");
    fflush(stdout);
#endif    
    /* IP address decayer. */
    pthread_create(&threadid[5], NULL, decay, (void *)5 );
#if TESTOUTPUT
    printf("Thread 5 created.\n");
    fflush(stdout);
#endif 

    /* Use join to keep the program alive.  If the listener thread dies, 
       I should die too. */
    
    pthread_join(threadid[0], (void **)status);
#if TESTOUTPUT
    puts("Thread 0 exited.");
#endif    
    
    printf("Exiting program.\n");
    return 0;
}
