/**********************************************************************
$Id: readusers.c,v 1.3 2002/09/09 22:27:41 bu Exp $

Copyright 2001-2002, Microbrew Software

Part of Microbrew Message Center
Multi-server Mailer Suite

http://www.microbrew.org/products/mmc/

by Bulent Yilmaz <bu@microbrew.org>
and James Brown <jim@microbrew.org>

Microbrew Message Center is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

Microbrew Message Center is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along with
Microbrew Message Center; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

**********************************************************************/

#include <stdio.h>
#include <mysql/mysql.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "confread.h"

/* Configuration file from autoconf. */
#include "../config.h"

#define ROOTBOXDIR PREFIX "/boxes"
#define VIRTFILE PREFIX "/etc/virtusertable"

#define DBCONF PREFIX "/etc/db.conf"
#define MAILSRVCONF PREFIX "/etc/mailsrv.conf"

int main() {

    MYSQL *dbs=NULL;
    MYSQL_RES *result=NULL;
    MYSQL_ROW myrow=NULL;
    char **domains=NULL;
    char **sdomains=NULL;
    char *dotmark=NULL;
    char *mydomain=NULL;
    int i=0,j=0,k=0;
    char buildquery[2048];
    char salt[3];
    char *dbfilename=NULL;
    struct stat filestat;
    char filename[2048];
    char tfilename[2048];
    FILE *authfile;
    FILE *virtuserfile;
    char *mysqlhost, *mysqluser, *mysqlpass;
    char *myhostname, *deliveryuser, *mailboxuid, *mailboxgid;
    int uid_i, gid_i;

    if((mysqlhost = readconf(DBCONF, "DBSERVER")) == NULL) {
        printf("Could not read 'DBSERVER' from %s.\n", DBCONF);
        return 1;
    }
                            
    if((mysqluser = readconf(DBCONF, "USERNAME")) == NULL) {
        printf("Could not read 'USERNAME' from %s.\n", DBCONF);
        return 2;
    }
                                                        
    if((mysqlpass = readconf(DBCONF, "PASSWORD")) == NULL) {
        printf("Could not read 'PASSWORD' from %s.\n", DBCONF);
        return 3;
    }
    
    if((myhostname = readconf(MAILSRVCONF, "HOSTNAME")) == NULL) {
        printf("Could not read 'HOSTNAME' from %s.\n", MAILSRVCONF);
        return 4;
    }
    
    if((deliveryuser = readconf(MAILSRVCONF, "DELIVERY")) == NULL) {
        printf("Could not read 'DELIVERY' from %s.\n", MAILSRVCONF);
        return 5;
    }
    
    if((mailboxuid = readconf(MAILSRVCONF, "MAILBOXUID")) == NULL) {
        printf("Could not read 'MAILBOXUID' from %s.\n", MAILSRVCONF);
        return 6;
    }
    
    uid_i = atoi(mailboxuid);
    
    if((mailboxgid = readconf(MAILSRVCONF, "MAILBOXGID")) == NULL) {
        printf("Could not read 'MAILBOXGID' from %s.\n", MAILSRVCONF);
        return 7;
    }
    
    gid_i = atoi(mailboxgid); 
    
    if((dbs = mysql_init(NULL))!=NULL) {
        if(mysql_real_connect(dbs, mysqlhost, mysqluser, mysqlpass, NULL, 0, NULL, 0)) {
            if(!mysql_select_db(dbs, "mmcadmin")) {
                if(!mysql_query(dbs, "SELECT pdomain,sdomains FROM customers")) {
                    result = mysql_use_result(dbs);
                    
                    while(myrow = mysql_fetch_row(result)) {
                        domains=realloc(domains, ((i+1)*sizeof(char *)) );
                        *(domains+i)=strdup(myrow[0]);
                        sdomains=realloc(sdomains, ((i+1)*sizeof(char *)) );
                        *(sdomains+i)=strdup(myrow[1]);
                        i++;
                    }
                    
                    if(domains) {
                        if(!mysql_select_db(dbs, "mmccust")) {
                            
                            if((virtuserfile = fopen(VIRTFILE, "w+")) != NULL) {

                                for(j=0;j<i;j++) {
                                    printf("%s: %s(%p)\n", *(domains+j), *(sdomains+j), *(sdomains+j));
                                    
                                    /* Check and create directories as necessary */
                                    snprintf(filename, 2048, "%s/%s", ROOTBOXDIR, *(domains+j));
                                    if(!stat(filename, &filestat)) {
                                        printf("'%s' already exists.  Skipping.\n",filename);
                                        
                                        /* Create entries in virtusertable for secondary domains. */
                                        
                                        if( strlen(*(sdomains+j)) ) {
                                            mydomain = strtok( *(sdomains+j), "\r\n, " );
                                            do {
                                                fprintf(virtuserfile,"@%s	\%1@%s\n", mydomain, *(domains+j));
                                            } while( (mydomain = strtok( NULL, "\r\n, " )) != NULL );
                                        } 
                                        
                                        snprintf(filename, 2048, "%s/%s/auth", ROOTBOXDIR, *(domains+j));
                                        if(stat(filename, &filestat)) {
                                            printf("'%s' does not exist.  Creating.\n",filename);
                                            if(mkdir(filename, 0x01FF) == 0) {  /* ~umask & o777 */ 
                                                chown(filename, uid_i, gid_i);
                                            } else printf("Could not create directory '%s'.\n", filename);
                                        } else printf("'%s' already exists.  Skipping.\n",filename);
                                        
                                        snprintf(filename, 2048, "%s/%s/mail", ROOTBOXDIR, *(domains+j));   
                                        if(stat(filename, &filestat)) {
                                            printf("'%s' does not exist.  Creating.\n",filename);  
                                            if(mkdir(filename, 0x01FF) == 0) {  /* ~umask & o777 */
                                                chown(filename, uid_i, gid_i); 
                                            } else printf("Could not create directory '%s'.\n", filename);
                                        } else printf("'%s' already exists.  Skipping.\n",filename);  
                                    } else {
                                        printf("'%s' does not exist.  Creating.\n",filename);
                                        
                                        if(mkdir(filename, 0x01FF) == 0) {  /* ~umask & o777 */
                                        
                                            /* Create entries in virtuserfile for secondary domains. */

                                            if( strlen(*(sdomains+j)) ) {
                                                mydomain = strtok( *(sdomains+j), "\r\n, " );
                                                do {
                                                    fprintf(virtuserfile,"@%s	\%1@%s\n", mydomain, *(domains+j));
                                                } while( (mydomain = strtok( NULL, "\r\n, " )) != NULL );
                                            } 

                                            /* Also create the other two directories */
                                            
                                            snprintf(filename, 2048, "%s/%s/auth", ROOTBOXDIR, *(domains+j));
                                            if(mkdir(filename, 0x01FF) == 0) {  /* ~umask & o777 */
                                                chown(filename, uid_i, gid_i);
                                            } else printf("Could not create directory '%s'.\n", filename);
                                            
                                            snprintf(filename, 2048, "%s/%s/mail", ROOTBOXDIR, *(domains+j));
                                            if(mkdir(filename, 0x01FF) == 0) {  /* ~umask & o777 */
                                                chown(filename, uid_i, gid_i);
                                            } else printf("Could not create directory '%s'.\n", filename);
                                        } else printf("Could not create directory '%s'.\n", filename);
                                    }

                                    dbfilename=strdup(*(domains+j));

                                    while( (dotmark = strchr(dbfilename, '.')) != NULL ) {
                                        *dotmark = '_';
                                    }
                                    
                                    snprintf(buildquery, 2048, "SELECT username, password, aliases FROM %s_users", dbfilename );
                                    if(!mysql_query(dbs, buildquery)) {
                                    
                                        result = mysql_use_result(dbs);
                                    
                                        while(myrow = mysql_fetch_row(result)) {
                                        
                                            snprintf(filename, 2048, "%s/%s/auth/%s", ROOTBOXDIR, *(domains+j), myrow[0]);
                                            
                                            if((authfile = fopen(filename, "w+")) != NULL) {
                                            
                                                srand(time(NULL));
                                            
                                                salt[0]=(rand() % 26) + 65;
                                                salt[1]=(rand() % 26) + 97;
                                                salt[2]=0;
                                            
                                                fprintf(authfile, "%s:%s:%s:%s:::\n", myrow[0], crypt(myrow[1], &salt[0]), mailboxuid, mailboxgid);
                                                fclose(authfile);
                                                chown(filename, uid_i, gid_i);
                                                
                                                /* Also create/chown the mailbox itself so that LDA can deliver. */
                                                
                                                snprintf(filename, 2048, "%s/%s/mail/%s", ROOTBOXDIR, *(domains+j), myrow[0]);
                                                if(!stat(filename, &filestat)) {
                                                    /* File exists, check ownership. */
                                                    
                                                    if(filestat.st_uid != uid_i) chown(filename, uid_i, gid_i);
                                                    else if(filestat.st_gid != gid_i) chown(filename, uid_i, gid_i);
                                                } else {
                                                    /* File does not exist.  Create. */
                                                    
                                                    if((authfile = fopen(filename, "w+")) != NULL) {
                                                        fclose(authfile);
                                                        chown(filename, uid_i, gid_i);
                                                    } else printf("Could not open mailbox file '%s'\n",filename);
                                                }
                                                
                                                fprintf(virtuserfile,"%s@%s	mmclda:%s@%s\n",myrow[0], *(domains+j), myrow[0], *(domains+j));
                                                if(strlen(myrow[2])) {
                                                    mydomain = strtok(myrow[2], "\r\n, ");
                                                    do {
                                                        fprintf(virtuserfile,"%s@%s	%s@%s\n", mydomain, *(domains+j), myrow[0], *(domains+j));
                                                    } while( (mydomain=strtok(NULL, "\r\n, ")) != NULL );
                                                }
                                            } else printf("Could not open auth file '%s'\n",filename);
                                        }
                                        
                                    } else puts("Could not query users list.");
                                    
                                    free(dbfilename);
                                } /* End of for loop. */
                            
                                fclose(virtuserfile);
                            } else puts("Could not open virtusertable file for writing.");
                        } else puts("Could not change domains.");
                    } else puts("No domains found.");
                } else puts("Customer lookup failed.");
            } else puts("Could not select the 'mmcadmin' database.");
        } else puts("Could not connect to the database.");
    
        mysql_close(dbs);    
    } else puts("Could not allocate memory for connect.");

    return 0;

}